require 'roar/json'
module Bankster
  module Representers
    module TransactionRepresenter
      include Roar::JSON
      collection_representer class: Bankster::Transaction

      property :purpose
      property :name
      property :iban
      property :bic

      # valutation_date is
      # - a Date object in ruby
      # - a ISO8601 date string in json
      property :valutation_date,
        reader: ->(doc:, represented:, **) { represented.valutation_date = Date.parse(doc['valutation_date']) if doc['valutation_date'] },
        writer: ->(doc:, represented:, **) { doc['valutation_date'] = represented.valutation_date.iso8601 if represented.valutation_date }

      # booking_date is
      # - a Date object in ruby
      # - a ISO8601 date string in json
      property :booking_date,
        reader: ->(doc:, represented:, **) { represented.booking_date = Date.parse(doc['booking_date']) if doc['booking_date'] },
        writer: ->(doc:, represented:, **) { doc['booking_date'] = represented.booking_date.iso8601 if represented.booking_date }

      # type is
      # - a symbol in ruby
      # - a string in json
      property :type,
        reader: ->(doc:, represented:, **) { represented.type = doc['type'].to_sym },
        writer: ->(doc:, represented:, **) { doc['type'] = represented.type.to_s }

      # amount is
      # - a Money object in ruby
      # - a Hash with "cents" and "currency" in json
      property :amount,
        reader: ->(doc:, represented:, **) {
          represented.amount = Money.new(doc['amount']['cents'], doc['amount']['currency'])
        },
        writer: ->(doc:, represented:, **) {
          doc['amount'] = {
            'cents' => represented.amount.cents.to_s,
            'currency' => represented.amount.currency
          }
        }
    end
  end
end
